"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rawInstance = exports.mockClass = exports.mockResolvedClass = void 0;
class MockedClassInstance {
}
;
/**
 * Creates a mock instance of a given class with customized method implementations,
 * where method return values are automatically unwrapped if they are promises.
 * @example
 * const userRepository = mockResolvedClass(UserRepository, {
 *   findUserByToken: { token: 'adhf753', name: 'John' },
 * });
 * @param Class - The class to mock.
 * @param givenMethods - An object containing customized key-value pairs where the key
 * is the method name and the value is the resolved method value.
 * @returns A mocked instance of the provided class with overridden methods.
 */
function mockResolvedClass(Class, givenMethods) {
    const MockedClass = rawInstance(Class);
    if (!givenMethods)
        return MockedClass;
    for (const classMethod in givenMethods) {
        const methodName = classMethod;
        const mockImplementation = () => givenMethods[methodName];
        MockedClass[methodName] = mockImplementation;
    }
    return MockedClass;
}
exports.mockResolvedClass = mockResolvedClass;
/**
 * Creates a mock instance of a given class with customized method implementations.
 * @example
 * const userRepository = mockClass(UserRepository, {
 *   findUserByToken: Promise.resolve({ token: 'adhf753', name: 'John' }),
 * });
 * @param Class - The class to mock.
 * @param methods - An object containing customized key-value pairs where the key
 * is the method name and the value is the returned method value.
 * @returns A mocked instance of the provided class with overridden methods.
 */
function mockClass(Class, givenMethods) {
    const MockedClass = rawInstance(Class);
    if (!givenMethods)
        return MockedClass;
    for (const classMethod in givenMethods) {
        const methodName = classMethod;
        const mockImplementation = () => givenMethods[methodName];
        MockedClass[methodName] = mockImplementation;
    }
    return MockedClass;
}
exports.mockClass = mockClass;
/**
 * Creates a raw mock instance of a given class with no method implementations.
 * This function is intended for use with testing frameworks like jest (spyOn) when
 * you want to set up **manual** method spies. Alternatively, consider using
 * {@link mockClass} or {@link mockResolvedClass} for more comprehensive method mocking.
 * @example
 * const rawUserRepository = rawInstance(UserRepository);
 * jest.spyOn(rawUserRepository, 'findUserByToken').mockRejectedValue(new Error('Token expired'));
 * @param Class - The class to mock.
 * @returns A raw mocked instance of the provided class with no implemented methods.
 */
function rawInstance(Class) {
    const MockedClass = new MockedClassInstance();
    const classMethods = Object.getOwnPropertyNames(Class.prototype).filter(methodName => methodName !== 'constructor' &&
        typeof Class.prototype[methodName] === 'function');
    for (const classMethod of classMethods) {
        const methodName = classMethod;
        const mockImplementation = () => {
            throw new Error(`Method ${classMethod} not implemented`);
        };
        MockedClass[methodName] = mockImplementation;
    }
    return MockedClass;
}
exports.rawInstance = rawInstance;
